@@ -1,7 +1,9 @@
-/* A Bison parser, made by GNU Bison 2.0.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
-/* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+/* Skeleton interface for Bison's Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,13 +17,21 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
-/* As a special exception, when this file is copied by Bison into a
-   Bison output file, you may use that output file without restriction.
-   This special exception was added by the Free Software Foundation
-   in version 1.24 of Bison.  */
+/* As a special exception, you may create a larger work that contains
+   part or all of the Bison parser skeleton and distribute that work
+   under terms of your choice, so long as that work isn't itself a
+   parser generator using the skeleton or a modified version thereof
+   as a parser skeleton.  Alternatively, if you modify or redistribute
+   the parser skeleton itself, you may (at your option) remove this
+   special exception, which will cause the skeleton and the resulting
+   Bison output files to be licensed under the GNU General Public
+   License without this special exception.
+
+   This special exception was added by the Free Software Foundation in
+   version 2.2 of Bison.  */
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -75,7 +85,7 @@
      LITERAL_STRING_T = 301,
      LITERAL_ARRAY_T = 302,
      SYMBOL_T = 303,
-     PIN_T = 304,
+     GPSIMOBJECT_T = 304,
      PORT_T = 305,
      EQU_T = 306,
      AND_T = 307,
@@ -123,6 +133,7 @@
      REG_T = 349
    };
 #endif
+/* Tokens.  */
 #define ABORT 258
 #define ATTACH 259
 #define BREAK 260
@@ -169,7 +180,7 @@
 #define LITERAL_STRING_T 301
 #define LITERAL_ARRAY_T 302
 #define SYMBOL_T 303
-#define PIN_T 304
+#define GPSIMOBJECT_T 304
 #define PORT_T 305
 #define EQU_T 306
 #define AND_T 307
@@ -219,9 +230,10 @@
 
 
 
-#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 138 "parse.yy"
-typedef union YYSTYPE {
+#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
+typedef union YYSTYPE
+#line 140 "parse.yy"
+{
   guint32              i;
   guint64             li;
   float                f;
@@ -237,17 +249,18 @@ typedef union YYSTYPE {
   Float*                    Float_P;
   Integer*                  Integer_P;
   String*                   String_P;
-  Value*                    Symbol_P;
-  Pin_t*                    Pin_P;
+  gpsimObject*              Symbol_P;
+  gpsimObject*              gpsimObject_P;
 
   StringList_t             *StringList_P;
   ExprList_t               *ExprList_P;
-  PinList_t                *PinList_P;
+  gpsimObjectList_t        *gpsimObjectList_P;
 
   Macro                    *Macro_P;
-} YYSTYPE;
-/* Line 1318 of yacc.c.  */
-#line 251 "parse.h"
+}
+/* Line 1489 of yacc.c.  */
+#line 263 "parse.h"
+	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -255,5 +268,3 @@ typedef union YYSTYPE {
 
 
 
-
-
