@@ -16,7 +16,7 @@ node_test p16f84 p18f452_ports p16f628 digital_stim analog_stim p12ce518 eeprom_
 interrupts_14bit macro_test logic_test resistor usart_test txisr_test 
 tmr0_16bit tmr1_16bit 
 switch_test p18f comparator a2d psp ttl ccp ccp_628 wavegen spi i2c port_stim 
-p12c509 wdt p12f675 p16f676'
+p12c509 wdt p12f675 p16f676 p16f690'
 
         echo ${DIRS}
         for i in ${DIRS} ; do
@@ -114,6 +114,8 @@ ${RT} comparator sim_628
 
 ${RT} comparator sim_877a
 
+${RT} comparator sim_882
+
 ${RT} comparator sim_10f204
 
 ${RT} a2d sim_p16c71
@@ -140,7 +142,9 @@ ${RT} psp sim_p18f452
 
 ${RT} psp sim_p16f871
 
-${RT} ttl sim_377
+${RT} ttl sim_ttl377
+
+${RT} ttl sim_ttl165
 
 ${RT} ccp sim_ccp_877a
 
@@ -148,6 +152,8 @@ ${RT} ccp sim_ccp_819
 
 ${RT} ccp sim_pwm_877a
 
+${RT} ccp sim_pwm_6520
+
 ${RT} ccp_628 sim
 
 ${RT} wavegen sim
@@ -166,19 +172,21 @@ ${RT} i2c sim_p16f876a
 
 ${RT} port_stim sim_port_stim
 
+${RT} p12f675	sim_p12f629
 ${RT} p12f675	sim_p12f675
-
 ${RT} p12f675	sim_p12f683
 
 ${RT} p12c509 sim_p12c509_reset
 
 ${RT} p16f84 sim_reset
 
+${RT} wdt sim_wdt_10f200
 ${RT} wdt sim_wdt_16f88
 ${RT} wdt sim_nwdt_16f88
 
 ${RT} wdt sim_wdt_16f628
 ${RT} wdt sim_nwdt_16f628
+${RT} wdt sim_nwdt_16f648a
 
 ${RT} wdt sim_wdt_16c64
 ${RT} wdt sim_nwdt_16c64
@@ -188,3 +196,4 @@ ${RT} wdt sim_nwdt_18f452
 
 ${RT} p16f676 sim_p16f676
 ${RT} p16f676 sim_reset
+${RT} p16f690 sim
