@@ -3,22 +3,22 @@
    Copyright (C) 2009 Roy R. Rankin
 
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include <stdio.h>
 #include <iostream>
@@ -139,7 +139,12 @@ void _14bit_processor::interrupt ()
 
   bp.clear_interrupt();
 
-  stack->push(pc->value);
+  if (bp.have_sleep()) {
+      bp.clear_sleep();
+      stack->push(pc->value+1);
+  } else {
+      stack->push(pc->value);
+  }
   intcon->clear_gie();
 
   pc->interrupt(INTERRUPT_VECTOR);
@@ -241,9 +246,6 @@ Pic14Bit::Pic14Bit(const char *_name, const char *_desc)
 //-------------------------------------------------------------------
 Pic14Bit::~Pic14Bit()
 {
-  //delete m_MCLR; <-- this is a package pin
-  delete m_MCLRMonitor;
-
   delete_sfr_register(m_portb);
   delete_sfr_register(m_trisb);
 
