@@ -235,7 +235,8 @@ _16bit_processor::_16bit_processor(const char *_name, const char *desc)
   m_lata  = new PicLatchRegister(this,"lata","", m_porta);
   m_lata->setEnableMask(0x7f);
 
-  m_portb = new PicPortBRegister(this,"portb","", &intcon, 8,0xff);
+  m_portb = new PicPortBRegister(this,"portb","", &intcon, 8,0xff, 
+	&intcon2, &intcon3);
   m_portb->assignRBPUSink(7,&intcon2);
   m_trisb = new PicTrisRegister(this,"trisb","", m_portb, false);
   m_latb  = new PicLatchRegister(this,"latb","", m_portb);
@@ -249,6 +250,7 @@ _16bit_processor::_16bit_processor(const char *_name, const char *desc)
   m_porta->addSink(&tmr0l,4);
 
   stack = new Stack16(this);
+  internal_osc = false;
 }
 
 //-------------------------------------------------------------------
@@ -291,26 +293,29 @@ void _16bit_processor :: create_sfr_map()
 
   add_sfr_register(m_porta,       0xf80,porv);
   add_sfr_register(m_portb,       0xf81,porv);
-  add_sfr_register(m_portc,       0xf82,porv);
+  if ( HasPortC() )
+    add_sfr_register(m_portc,       0xf82,porv);
 
   add_sfr_register(m_lata,        0xf89,porv);
   add_sfr_register(m_latb,        0xf8a,porv);
-  add_sfr_register(m_latc,        0xf8b,porv);
+  if ( HasPortC() )
+    add_sfr_register(m_latc,        0xf8b,porv);
 
   add_sfr_register(m_trisa,       0xf92,RegisterValue(0x7f,0));
   add_sfr_register(m_trisb,       0xf93,RegisterValue(0xff,0));
-  add_sfr_register(m_trisc,       0xf94,RegisterValue(0xff,0));
+  if ( HasPortC() )
+    add_sfr_register(m_trisc,       0xf94,RegisterValue(0xff,0));
 
   add_sfr_register(&pie1,	  0xf9d,porv,"pie1");
   add_sfr_register(&pir1,	  0xf9e,porv,"pir1");
-
   add_sfr_register(&ipr1,	  0xf9f,porv,"ipr1");
+
   add_sfr_register(&pie2,	  0xfa0,porv,"pie2");
   add_sfr_register(&pir2,	  0xfa1,porv,"pir2");
   add_sfr_register(&ipr2,	  0xfa2,porv,"ipr2");
 
 
-  usart.initialize(&pir_set_def,&(*m_portc)[6], &(*m_portc)[7],
+  usart.initialize(&pir1,&(*m_portc)[6], &(*m_portc)[7],
 		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
                    new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
@@ -324,9 +329,12 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&tmr3l,	  0xfb2,porv,"tmr3l");
   add_sfr_register(&tmr3h,	  0xfb3,porv,"tmr3h");
 
-  add_sfr_register(&ccp2con,	  0xfba,porv,"ccp2con");
-  add_sfr_register(&ccpr2l,	  0xfbb,porv,"ccpr2l");
-  add_sfr_register(&ccpr2h,	  0xfbc,porv,"ccpr2h");
+  if ( HasCCP2() )
+  {
+    add_sfr_register(&ccp2con,	  0xfba,porv,"ccp2con");
+    add_sfr_register(&ccpr2l,	  0xfbb,porv,"ccpr2l");
+    add_sfr_register(&ccpr2h,	  0xfbc,porv,"ccpr2h");
+  }
   add_sfr_register(&ccp1con,	  0xfbd,porv,"ccp1con");
   add_sfr_register(&ccpr1l,	  0xfbe,porv,"ccpr1l");
   add_sfr_register(&ccpr1h,	  0xfbf,porv,"ccpr1h");
@@ -422,6 +430,8 @@ void _16bit_processor :: create_sfr_map()
   if (e) {
     add_sfr_register(e->get_reg_eedata(), 0xfa8);
     add_sfr_register(e->get_reg_eeadr(), 0xfa9);
+    if ( e->get_reg_eeadrh() )
+        add_sfr_register(e->get_reg_eeadrh(), 0xfaa);
     add_sfr_register(e->get_reg_eecon1(), 0xfa6, RegisterValue(0,0));
     add_sfr_register(e->get_reg_eecon2(), 0xfa7);
   }
@@ -445,8 +455,8 @@ void _16bit_processor :: create_sfr_map()
   tmr2.pir_set = &pir_set_def; //get_pir_set();
   tmr2.pr2    = &pr2;
   tmr2.t2con  = &t2con;
-  tmr2.ccp1con = &ccp1con;
-  tmr2.ccp2con = &ccp2con;
+  tmr2.add_ccp ( &ccp1con );
+  tmr2.add_ccp ( &ccp2con );
   pr2.tmr2    = &tmr2;
 
 
@@ -462,7 +472,7 @@ void _16bit_processor :: create_sfr_map()
   t3con.ccpr1l = &ccpr1l;
   t3con.ccpr2l = &ccpr2l;
 
-  ccp1con.setCrosslinks(&ccpr1l, &pir1, &tmr2);
+  ccp1con.setCrosslinks(&ccpr1l, &pir1, PIR1v2::CCP1IF, &tmr2);
   ccp1con.setIOpin(&((*m_portc)[2]));
   ccpr1l.ccprh  = &ccpr1h;
   ccpr1l.tmrl   = &tmr1l;
