@@ -85,6 +85,9 @@ void INTCON::peripheral_interrupt ( bool hi_pri )
   if ( hi_pri )
       cout << "Dodgy call to 14-bit INTCON::peripheral_interrupt with priority set\n";
 
+  if (cpu_pic->is_sleeping())
+        cpu_pic->exit_sleep();
+
   if(  (value.get() & (GIE | XXIE)) == (GIE | XXIE) )
     cpu_pic->BP_set_interrupt();
 }
@@ -136,6 +139,24 @@ INTCON3::INTCON3(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc)
 {}
 
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+void INTCON3::put_value(unsigned int new_value)
+{
+  unsigned int old_value = value.get();
+  value.put(new_value);
+
+
+}
+
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+void INTCON3::put(unsigned int new_value)
+{
+  trace.raw(write_trace.get() | value.get());
+  put_value(new_value);
+}
+
 //----------------------------------------------------------------------
 //----------------------------------------------------------------------
 INTCON_14_PIR::INTCON_14_PIR(Processor *pCpu, const char *pName, const char *pDesc)
