@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 //-------------------------------------------------------------------
 //                     interface.cc
@@ -356,7 +356,6 @@ void start_run_thread(void)
 void gpsimInterface::start_simulation (double duration)
 {
 
-#if defined(CLOCK_EXPERIMENTS)
 
   Processor *cpu = get_active_cpu();
   if (cpu) {
@@ -368,44 +367,6 @@ void gpsimInterface::start_simulation (double duration)
     simulation_has_stopped();
   }
 
-#else
-  Processor *cpu = get_active_cpu();
-
-  mbSimulating = true;
-
-  if(cpu) {
-#if GLIB_MAJOR_VERSION >= 2
-
-    if(gUsingThreads()) {
-      static bool thread_initialized=false;
-
-      if(!thread_initialized) {
-	start_run_thread();
-	g_usleep(10000);
-	thread_initialized = true;
-      }
-
-      g_mutex_lock (muRunMutex);
-
-      tcpu = cpu;
-      printf("signalling run thread\n");
-      g_cond_signal  (cvRunCondition);
-      g_mutex_unlock (muRunMutex);
-      printf("leaving start_simulation\n");
-    } else
-#endif
-      {
-
-	if(verbosity && verbosity->getVal()) {
-	  cout << "running...\n";
-	  cpu->run(true);
-	} else
-	  cpu->run(false);
-      }
-  }
-
-  mbSimulating = false;
-#endif
 }
 
 void gpsimInterface::step_simulation (int nSteps)
@@ -576,18 +537,20 @@ void  gpsimInterface::remove_interface  (unsigned int interface_id)
 
 void  gpsimInterface::set_update_rate  (guint64 _update_rate)
 {
-
-  guint64 fc = get_cycles().get() + _update_rate;
-
   update_rate = _update_rate;
 
-  if(fc) {
-    if(future_cycle)
-      get_cycles().reassign_break(future_cycle, fc, this);
-    else
-      get_cycles().set_break(fc, this);
+  if(update_rate)
+  {
+    guint64 fc = get_cycles().get() + _update_rate;
+
+    if(fc) {
+      if(future_cycle)
+        get_cycles().reassign_break(future_cycle, fc, this);
+      else
+        get_cycles().set_break(fc, this);
   
-    future_cycle = fc;
+      future_cycle = fc;
+    }
   }
 }
 
