@@ -384,7 +384,7 @@ void P12F629::exit_sleep()
 void P12F629::option_new_bits_6_7(unsigned int bits)
 {
 	Dprintf(("P12F629::option_new_bits_6_7 bits=%x\n", bits));
-  m_gpio->setIntEdge ( (bits & OPTION_REG::BIT6) != OPTION_REG::BIT6); 
+  m_gpio->setIntEdge ( (bits & OPTION_REG::BIT6) == OPTION_REG::BIT6); 
   m_wpu->set_wpu_pu ( (bits & OPTION_REG::BIT7) != OPTION_REG::BIT7); 
 }
 //========================================================================
@@ -530,11 +530,11 @@ void P12F683::create_sfr_map()
   tmr2.pir_set   = get_pir_set();
   tmr2.pr2    = &pr2;
   tmr2.t2con  = &t2con;
-  tmr2.ccp1con = &ccp1con;
+  tmr2.add_ccp ( &ccp1con );
   pr2.tmr2    = &tmr2;
 
 
-  ccp1con.setCrosslinks(&ccpr1l, pir1, &tmr2);
+  ccp1con.setCrosslinks(&ccpr1l, pir1, PIR1v1::CCP1IF, &tmr2);
   ccp1con.setIOpin(&((*m_gpio)[2]));
   ccpr1l.ccprh  = &ccpr1h;
   ccpr1l.tmrl   = &tmr1l;
