@@ -292,12 +292,12 @@ void P16X6X_processor::create_sfr_map()
   tmr2.pir_set   = get_pir_set();
   tmr2.pr2    = &pr2;
   tmr2.t2con  = &t2con;
-  tmr2.ccp1con = &ccp1con;
-  tmr2.ccp2con = &ccp2con;
+  tmr2.add_ccp ( &ccp1con );
+  tmr2.add_ccp ( &ccp2con );
   pr2.tmr2    = &tmr2;
 
 
-  ccp1con.setCrosslinks(&ccpr1l, pir1, &tmr2);
+  ccp1con.setCrosslinks(&ccpr1l, pir1, PIR1v1::CCP1IF, &tmr2);
   ccp1con.setIOpin(&((*m_portc)[2]));
   ccpr1l.ccprh  = &ccpr1h;
   ccpr1l.tmrl   = &tmr1l;
@@ -468,13 +468,13 @@ void P16C63::create_sfr_map(void)
   // get_pir_set()->set_pir2(get_pir2());
   pir_set_def.set_pir2(pir2);
 
-  ccp2con.setCrosslinks(&ccpr2l, pir2, &tmr2);
+  ccp2con.setCrosslinks(&ccpr2l, pir2, PIR2v1::CCP2IF, &tmr2);
   ccp2con.setIOpin(&((*m_portc)[1]));
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
   ccpr2h.ccprl  = &ccpr2l;
 
-  usart.initialize(get_pir_set(),&(*m_portc)[6], &(*m_portc)[7],
+  usart.initialize(pir1,&(*m_portc)[6], &(*m_portc)[7],
                    new _TXREG(this,"txreg", "USART Transmit Register", &usart),
                    new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
@@ -687,14 +687,14 @@ void P16C65::create_sfr_map(void)
   pir_set_def.set_pir2(pir2);
 
 
-  ccp2con.setCrosslinks(&ccpr2l, pir2, &tmr2);
+  ccp2con.setCrosslinks(&ccpr2l, pir2, PIR2v2::CCP2IF, &tmr2);
   ccp2con.setIOpin(&((*m_portc)[1]));
 
   ccpr2l.ccprh  = &ccpr2h;
   ccpr2l.tmrl   = &tmr1l;
   ccpr2h.ccprl  = &ccpr2l;
 
-  usart.initialize(get_pir_set(),&(*m_portc)[6], &(*m_portc)[7],
+  usart.initialize(pir1,&(*m_portc)[6], &(*m_portc)[7],
                    new _TXREG(this,"txreg", "USART Transmit Register", &usart),
                    new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
