@@ -252,12 +252,24 @@ ProcessorConstructor pP16F628A(P16F628::construct ,
                               "__16F628A",  "pic16f628a",   "p16f628a", "16f628a");
 ProcessorConstructor pP16F630(P16F630::construct ,
                               "__16F630",   "pic16f630",    "p16f630",  "16f630");
+ProcessorConstructor pP16F631(P16F631::construct ,
+                              "__16F631",   "pic16f631",    "p16f631",  "16f631");
 ProcessorConstructor pP16F648(P16F648::construct ,
                               "__16F648",   "pic16f648",    "p16f648",  "16f648");
 ProcessorConstructor pP16F648A(P16F648::construct ,
                               "__16F648A",  "pic16f648a",   "p16f648a", "16f648a");
 ProcessorConstructor pP16F676(P16F676::construct ,
                               "__16F676",   "pic16f676",    "p16f676",  "16f676");
+ProcessorConstructor pP16F677(P16F677::construct ,
+                              "__16F677",   "pic16f677",    "p16f677",  "16f677");
+ProcessorConstructor pP16F685(P16F685::construct ,
+                              "__16F685",   "pic16f685",    "p16f685",  "16f685");
+ProcessorConstructor pP16F687(P16F687::construct ,
+                              "__16F687",   "pic16f687",    "p16f687",  "16f687");
+ProcessorConstructor pP16F689(P16F689::construct ,
+                              "__16F689",   "pic16f689",    "p16f689",  "16f689");
+ProcessorConstructor pP16F690(P16F690::construct ,
+                              "__16F690",   "pic16f690",    "p16f690",  "16f690");
 ProcessorConstructor pP16F818(P16F818::construct ,
                               "__16F818",   "pic16f818",    "p16f818",  "16f818");
 ProcessorConstructor pP16F819(P16F819::construct ,
@@ -320,14 +332,30 @@ ProcessorConstructor pP18F1220(P18F1220::construct,
                               "__18F1220",  "pic18f1220",   "p18f1220", "18f1220");
 ProcessorConstructor pP18F1320(P18F1320::construct,
                               "__18F1320",  "pic18f1320",   "p18f1320", "18f1320");
+ProcessorConstructor pP18F2221(P18F2221::construct,
+                              "__18F2221",  "pic18f2221",   "p18f2221", "18f2221");
 ProcessorConstructor pP18F2321(P18F2321::construct,
                               "__18F2321",  "pic18f2321",   "p18f2321", "18f2321");
+ProcessorConstructor pP18F2420(P18F2420::construct,
+                              "__18F2420",  "pic18f2420",   "p18f2420", "18f2420");
+ProcessorConstructor pP18F2520(P18F2520::construct,
+                              "__18F2520",  "pic18f2520",   "p18f2520", "18f2520");
 ProcessorConstructor pP18F2455(P18F2455::construct,
                               "__18F2455",  "pic18f2455",   "p18f2455", "18f2455");
+ProcessorConstructor pP18F4221(P18F4221::construct,
+                              "__18F4221",  "pic18f4221",   "p18f4221", "18f4221");
 ProcessorConstructor pP18F4321(P18F4321::construct,
                               "__18F4321",  "pic18f4321",   "p18f4321", "18f4321");
+ProcessorConstructor pP18F4420(P18F4420::construct,
+                              "__18F4420",  "pic18f4420",   "p18f4420", "18f4420");
+ProcessorConstructor pP18F4520(P18F4520::construct,
+                              "__18F4520",  "pic18f4520",   "p18f4520", "18f4520");
 ProcessorConstructor pP18F4455(P18F4455::construct,
                               "__18F4455",  "pic18f4455",   "p18f4455", "18f4455");
+ProcessorConstructor pP18F4620(P18F4620::construct,
+                              "__18F4620",  "pic18f4620",   "p18f4620", "18f4620");
+ProcessorConstructor pP18F6520(P18F6520::construct,
+                              "__18F6520",  "pic18f6520",   "p18f6520", "18f6520");
 
 
 //========================================================================
@@ -425,6 +453,7 @@ void pic_processor::sleep ()
 
 void pic_processor::enter_sleep()
 {
+
   status->put_TO(1);
   status->put_PD(0);
 
@@ -1298,6 +1327,7 @@ bool pic_processor::LoadProgramFile(const char *pFilename, FILE *pFile,
     fseek(pFile, 0, SEEK_SET);
     iReturn = aFileTypes[1]->LoadProgramFile(&pProcessor, pFilename, pFile, pProcessorName);
   }
+  cout << "Leaving pic_processor::LoadProgramFile\n";
   return iReturn == ProgramFileType::SUCCESS;
 }
 
@@ -1528,7 +1558,7 @@ void WDT::callback()
 
     if(breakpoint)
       bp.halt();
-    else if (cpu->is_sleeping())
+    else if (cpu->is_sleeping() && cpu->exit_wdt_sleep())
     {
         cout << "WDT expired during sleep\n";
         update();
