@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998-2000 Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #include <stdio.h>
@@ -33,7 +33,7 @@ Boston, MA 02111-1307, USA.  */
 
 //#define DEBUG
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("0x%06X %s() ",cycles.get(),__FUNCTION__); printf arg; }
+#define Dprintf(arg) {printf("0x%06"PRINTF_GINT64_MODIFIER"X %s() ",cycles.get(),__FUNCTION__); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
@@ -53,7 +53,6 @@ Program_Counter::Program_Counter(const char *name, const char *desc, Module *pM)
 
   reset_address = 0;
   value = 0;
-  memory_size_mask = 0;
   pclath_mask = 0x1800;    // valid pclath bits for branching in 14-bit cores 
   instruction_phase = 0;
 
@@ -96,7 +95,18 @@ void Program_Counter::increment()
 
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_increment | value);
-  value = (value + 1) & memory_size_mask;
+  value = (value + 1);
+  if (value == memory_size) // Some processors start at highest memory and roll over
+  {
+	
+        printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, value, memory_size);
+	value = 0;
+  }
+  else if (value > memory_size) // assume this is a mistake
+  {
+        printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, value, memory_size);
+        bp.halt();
+  }
 
   // Update pcl. Note that we don't want to pcl.put() because that 
   // will trigger a break point if there's one set on pcl. (A read/write
@@ -105,11 +115,7 @@ void Program_Counter::increment()
 
   cpu_pic->pcl->value.put(value & 0xff);
 
-#ifdef CLOCK_EXPERIMENTS
   mCurrentPhase->setNextPhase(mExecute1Cycle);
-#else
-  get_cycles().increment();
-#endif
 }
 
 //--------------------------------------------------
@@ -125,23 +131,14 @@ void Program_Counter::skip()
   trace.raw(trace_skip | value);
 
 
-#ifdef CLOCK_EXPERIMENTS
-  mExecute2ndHalf->firstHalf((value + 2) & memory_size_mask);
-#else
-  //value = (value + 1) & memory_size_mask;
-  //cpu_pic->pcl->value.put(value & 0xff);
-  //get_cycles().increment();
 
-  value = (value + 1) & memory_size_mask;
-  cpu_pic->pcl->value.put(value & 0xff);
-  get_cycles().increment();
-
-  trace.raw(trace_increment | value);
-
-  value = (value + 1) & memory_size_mask;
-  cpu_pic->pcl->value.put(value & 0xff);
-  get_cycles().increment();
-#endif
+  if ((value + 2) >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, value, memory_size);
+    bp.halt();
+  }
+  else
+    mExecute2ndHalf->firstHalf( value + 2);
 
 }
 
@@ -155,7 +152,6 @@ void Program_Counter::start_skip()
 }
 
 //========================================================================
-#if defined(CLOCK_EXPERIMENTS)
 
 phaseExecute2ndHalf::phaseExecute2ndHalf(Processor *pcpu)
   : ProcessorPhase(pcpu), m_uiPC(0)
@@ -206,7 +202,6 @@ ClockPhase *phaseExecuteInterrupt::advance()
   get_cycles().increment();
   return m_pNextPhase;
 }
-#endif // defined(CLOCK_EXPERIMENTS)
 
 //--------------------------------------------------
 // jump - update the program counter. All branching instructions except computed gotos
@@ -225,14 +220,13 @@ void Program_Counter::jump(unsigned int new_address)
 
   // see Update pcl comment in Program_Counter::increment()
   
-#ifdef CLOCK_EXPERIMENTS
-  mExecute2ndHalf->firstHalf(new_address & memory_size_mask);
-#else
-  value = new_address & memory_size_mask;
-  cpu_pic->pcl->value.put(value & 0xff);
-  get_cycles().increment();
-  get_cycles().increment();
-#endif
+  if (new_address >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, new_address, memory_size);
+    bp.halt();
+  }
+  else
+    mExecute2ndHalf->firstHalf(new_address);
 
 }
 
@@ -244,26 +238,16 @@ void Program_Counter::interrupt(unsigned int new_address)
 {
   Dprintf(("PC=0x%x 0x%x\n",value,new_address));
 
-#ifdef CLOCK_EXPERIMENTS
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_branch | value);
 
-  mExecuteInterrupt->firstHalf(new_address & memory_size_mask);
-#else
-
-  // Trace the value of the program counter before it gets changed.
-  trace.raw(trace_branch | value);
-
-  // Use the new_address and the cached pclath (or page select bits for 12 bit cores)
-  // to generate the destination address:
-
-  value = new_address & memory_size_mask;
-
-  cpu_pic->pcl->value.put(value & 0xff);    // see Update pcl comment in Program_Counter::increment()
-  
-  get_cycles().increment();
-  get_cycles().increment();
-#endif
+  if (new_address >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, new_address, memory_size);
+    bp.halt();
+  }
+  else
+    mExecute2ndHalf->firstHalf(new_address);
 
 }
 
@@ -283,7 +267,12 @@ void Program_Counter::computed_goto(unsigned int new_address)
   // Use the new_address and the cached pclath (or page select bits for 12 bit cores)
   // to generate the destination address:
 
-  value = (new_address | cpu_pic->get_pclath_branching_modpcl() ) & memory_size_mask;
+  value = new_address | cpu_pic->get_pclath_branching_modpcl() ;
+  if (value >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, value, memory_size);
+    bp.halt();
+  }
 
   // see Update pcl comment in Program_Counter::increment()
 
@@ -297,11 +286,7 @@ void Program_Counter::computed_goto(unsigned int new_address)
   // the instruction (i.e. via the ::increment() method). The second cycle occurs
   // here:
 
-#ifdef CLOCK_EXPERIMENTS
   mCurrentPhase = mExecute1Cycle;
-#else
-  get_cycles().increment();
-#endif
 }
 
 //--------------------------------------------------
@@ -315,18 +300,13 @@ void Program_Counter::new_address(unsigned int new_address)
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_branch | value);
 
-#ifdef CLOCK_EXPERIMENTS
-  mExecute2ndHalf->firstHalf(new_address & memory_size_mask);
-#else
-  value = new_address & memory_size_mask;
-
-  // see Update pcl comment in Program_Counter::increment()
-
-  cpu_pic->pcl->value.put(value & 0xff);
-
-  get_cycles().increment();
-  get_cycles().increment();
-#endif
+  if (new_address >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, new_address, memory_size);
+    bp.halt();
+  }
+  else
+    mExecute2ndHalf->firstHalf(new_address);
 }
 
 //--------------------------------------------------
@@ -335,8 +315,14 @@ void Program_Counter::new_address(unsigned int new_address)
 
 unsigned int Program_Counter::get_next()
 {
+  unsigned int new_address = value + cpu_pic->program_memory[value]->instruction_size();
 
-  return( (value + cpu_pic->program_memory[value]->instruction_size()) & memory_size_mask);
+  if (new_address >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, new_address, memory_size);
+    bp.halt();
+  }
+  return( new_address);
 
 }
 
@@ -353,7 +339,12 @@ void Program_Counter::put_value(unsigned int new_value)
 
   trace.raw(trace_other | value);
 
-  value = new_value & memory_size_mask;
+  if (new_value >= memory_size)
+  {
+    printf("%s PC=0x%x >= memory size 0x%x\n", __FUNCTION__, new_value, memory_size);
+    bp.halt();
+  }
+  value = new_value;
   cpu_pic->pcl->value.put(value & 0xff);
   cpu_pic->pclath->value.put((new_value >> 8) & PCLATH_MASK);
 
@@ -366,9 +357,8 @@ void Program_Counter::reset()
 { 
   //trace.program_counter(value);  //FIXME
   value = reset_address;
-#ifdef CLOCK_EXPERIMENTS
-  mExecute2ndHalf->firstHalf(reset_address & memory_size_mask);
-#endif
+  value = (value >= memory_size) ? value - memory_size : value;
+  mExecute2ndHalf->firstHalf(value);
 }
 
 
