@@ -141,7 +141,7 @@ _TXSTA::~_TXSTA()
 
 //-----------------------------------------------------------
 _RCREG::_RCREG(Processor *pCpu, const char *pName, const char *pDesc, USART_MODULE *pUSART)
-  : sfr_register(pCpu, pName, pDesc), mUSART(pUSART), m_rcsta(0)
+  : sfr_register(pCpu, pName, pDesc), fifo_sp(0),  mUSART(pUSART), m_rcsta(0)
 {
   assert(mUSART);
 }
@@ -160,7 +160,8 @@ _BAUDCON::_BAUDCON(Processor *pCpu, const char *pName, const char *pDesc)
 
 _SPBRG::_SPBRG(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    txsta(0), rcsta(0), brgh(0), baudcon(0), skip(0)
+    txsta(0), rcsta(0), brgh(0), baudcon(0), start_cycle(0), last_cycle(0),
+    future_cycle(0), skip(0)
 {
 }
 
@@ -1106,13 +1107,13 @@ void _BAUDCON::put(unsigned int new_value)
 //--------------------------------------------------
 // member functions for the USART
 //--------------------------------------------------
-void USART_MODULE::initialize(PIR_SET *_pir_set,
+void USART_MODULE::initialize(PIR *_pir,
                               PinModule *tx_pin, PinModule *rx_pin,
                               _TXREG *_txreg, _RCREG *_rcreg)
 {
   assert(_txreg && _rcreg);
 
-  pir_set = _pir_set;
+  pir = _pir;
 
   spbrg.txsta = &txsta;
   spbrg.rcsta = &rcsta;
@@ -1137,42 +1138,42 @@ void USART_MODULE::initialize(PIR_SET *_pir_set,
 
 bool USART_MODULE::bIsTXempty()
 {
-  return pir_set ? pir_set->get_txif() : true;
+  return pir ? pir->get_txif() : true;
 }
 void USART_MODULE::emptyTX()
 {
   Dprintf(("usart::empty - setting TXIF\n"));
-  if (rcsta.bSPEN() && txsta.bTXEN() && pir_set)
-    pir_set->set_txif();
+  if (rcsta.bSPEN() && txsta.bTXEN() && pir)
+    pir->set_txif();
 
 }
 
 void USART_MODULE::full()
 {
   Dprintf(("txreg::full - clearing TXIF\n"));
-  if(pir_set)
-    pir_set->clear_txif();
+  if(pir)
+    pir->clear_txif();
 }
 
 void USART_MODULE::set_rcif()
 {
   Dprintf((" - setting RCIF\n"));
-  if(pir_set)
-    pir_set->set_rcif();
+  if(pir)
+    pir->set_rcif();
 }
 
 void USART_MODULE::clear_rcif()
 {
   Dprintf((" - clearing RCIF\n"));
-  if(pir_set)
-    pir_set->clear_rcif();
+  if(pir)
+    pir->clear_rcif();
 }
 
 //--------------------------------------------------
 USART_MODULE::USART_MODULE(Processor *pCpu)
-  : txsta(pCpu,"txsta","USART Transmit Status",this),
-    rcsta(pCpu,"rcsta","USART Receive Status",this),
-    spbrg(pCpu,"spbrg","Serial Port Baud Rate Generator"),
+  : txsta(pCpu,"","USART Transmit Status",this),    // Don't set names incase there are two UARTS
+    rcsta(pCpu,"","USART Receive Status",this),
+    spbrg(pCpu,"","Serial Port Baud Rate Generator"),
     spbrgh(pCpu,"spbrgh","Serial Port Baud Rate high byte"),
     baudcon(pCpu,"baudcon","Serial Port Baud Rate Control"),
     is_eusart(false)
